@@ -6,6 +6,7 @@
 #include <stdint.h>
 
 #include "c.h"
+#include "rpmatch.h"
 #include "fdisk.h"
 #include "pt-sun.h"
 #include "pt-mbr.h"
@@ -89,7 +90,7 @@ DECLARE_MENU_CB(generic_menu_cb);
 #define MENU_XENT_NEST(k, t, l, p)	{ .title = t, .key = k, .expert = 1, .label = l, .parent = p }
 
 /* Generic menu */
-struct menu menu_generic = {
+static const struct menu menu_generic = {
 	.callback	= generic_menu_cb,
 	.entries	= {
 		MENU_BSEP(N_("Generic")),
@@ -127,7 +128,7 @@ struct menu menu_generic = {
 	}
 };
 
-struct menu menu_createlabel = {
+static const struct menu menu_createlabel = {
 	.callback = createlabel_menu_cb,
 	.exclude = FDISK_DISKLABEL_BSD,
 	.nonested = 1,
@@ -145,7 +146,7 @@ struct menu menu_createlabel = {
 	}
 };
 
-struct menu menu_geo = {
+static const struct menu menu_geo = {
 	.callback = geo_menu_cb,
 	.exclude = FDISK_DISKLABEL_GPT | FDISK_DISKLABEL_BSD,
 	.entries = {
@@ -157,7 +158,7 @@ struct menu menu_geo = {
 	}
 };
 
-struct menu menu_gpt = {
+static const struct menu menu_gpt = {
 	.callback = gpt_menu_cb,
 	.label = FDISK_DISKLABEL_GPT,
 	.entries = {
@@ -165,6 +166,7 @@ struct menu menu_gpt = {
 		MENU_XENT('i', N_("change disk GUID")),
 		MENU_XENT('n', N_("change partition name")),
 		MENU_XENT('u', N_("change partition UUID")),
+		MENU_XENT('l', N_("change table length")),
 		MENU_XENT('M', N_("enter protective/hybrid MBR")),
 
 		MENU_XSEP(""),
@@ -177,7 +179,7 @@ struct menu menu_gpt = {
 	}
 };
 
-struct menu menu_sun = {
+static const struct menu menu_sun = {
 	.callback = sun_menu_cb,
 	.label = FDISK_DISKLABEL_SUN,
 	.entries = {
@@ -194,7 +196,7 @@ struct menu menu_sun = {
 	}
 };
 
-struct menu menu_sgi = {
+static const struct menu menu_sgi = {
 	.callback = sgi_menu_cb,
 	.label = FDISK_DISKLABEL_SGI,
 	.entries = {
@@ -207,7 +209,7 @@ struct menu menu_sgi = {
 	}
 };
 
-struct menu menu_dos = {
+static const struct menu menu_dos = {
 	.callback = dos_menu_cb,
 	.label = FDISK_DISKLABEL_DOS,
 	.entries = {
@@ -225,7 +227,7 @@ struct menu menu_dos = {
 	}
 };
 
-struct menu menu_bsd = {
+static const struct menu menu_bsd = {
 	.callback = bsd_menu_cb,
 	.label = FDISK_DISKLABEL_BSD,
 	.entries = {
@@ -422,11 +424,24 @@ int process_fdisk_menu(struct fdisk_context **cxt0)
 		prompt = _("Command (m for help): ");
 
 	fputc('\n',stdout);
-	rc = get_user_reply(cxt, prompt, buf, sizeof(buf));
-	if (rc)
+	rc = get_user_reply(prompt, buf, sizeof(buf));
+
+	if (rc == -ECANCELED) {
+		/* Map ^C and ^D in main menu to 'q' */
+		if (is_interactive
+		    && fdisk_label_is_changed(fdisk_get_label(cxt, NULL))) {
+			rc = get_user_reply(
+				_("\nDo you really want to quit? "),
+				buf, sizeof(buf));
+			if (rc || !rpmatch(buf))
+				return 0;
+		}
+		key = 'q';
+	} else if (rc) {
 		return rc;
+	} else
+		key = buf[0];
 
-	key = buf[0];
 	ent = get_fdisk_menu_entry(cxt, key, &menu);
 	if (!ent) {
 		fdisk_warnx(cxt, _("%c: unknown command"), key);
@@ -464,9 +479,15 @@ static int script_read(struct fdisk_context *cxt)
 		fdisk_warn(cxt, _("Cannot open %s"), filename);
 	else if (!sc)
 		fdisk_warnx(cxt, _("Failed to parse script file %s"), filename);
-	else if (fdisk_apply_script(cxt, sc) != 0)
+	else if (fdisk_apply_script(cxt, sc) != 0) {
 		fdisk_warnx(cxt, _("Failed to apply script %s"), filename);
-	else
+		fdisk_warnx(cxt, _("Resetting fdisk!"));
+		rc = fdisk_reassign_device(cxt);
+                if (rc == 0 && !fdisk_has_label(cxt)) {
+                        fdisk_info(cxt, _("Device does not contain a recognized partition table."));
+                        fdisk_create_disklabel(cxt, NULL);
+		}
+	} else
 		fdisk_info(cxt, _("Script successfully applied."));
 
 	fdisk_unref_script(sc);
@@ -516,6 +537,37 @@ done:
 	return rc;
 }
 
+static int ask_for_wipe(struct fdisk_context *cxt, size_t partno)
+{
+	struct fdisk_partition *tmp = NULL;
+	char *fstype = NULL;
+	int rc, yes = 0;
+
+	rc = fdisk_get_partition(cxt, partno, &tmp);
+	if (rc)
+		goto done;
+
+	rc = fdisk_partition_to_string(tmp, cxt, FDISK_FIELD_FSTYPE, &fstype);
+	if (rc || fstype == NULL)
+		goto done;
+
+	fdisk_warnx(cxt, _("Partition #%zu contains a %s signature."), partno + 1, fstype);
+
+	if (pwipemode == WIPEMODE_AUTO && isatty(STDIN_FILENO))
+		fdisk_ask_yesno(cxt, _("Do you want to remove the signature?"), &yes);
+	else if (pwipemode == WIPEMODE_ALWAYS)
+		yes = 1;
+
+	if (yes) {
+		fdisk_info(cxt, _("The signature will be removed by a write command."));
+		rc = fdisk_wipe_partition(cxt, partno, TRUE);
+	}
+done:
+	fdisk_unref_partition(tmp);
+	free(fstype);
+	return rc;
+}
+
 /*
  * Basic fdisk actions
  */
@@ -544,7 +596,11 @@ static int generic_menu_cb(struct fdisk_context **cxt0,
 		if (fdisk_get_parent(cxt))
 			break; /* nested PT, don't leave */
 		fdisk_info(cxt, _("The partition table has been altered."));
-		rc = fdisk_reread_partition_table(cxt);
+
+		if (device_is_used)
+			rc = fdisk_reread_changes(cxt, original_layout);
+		else
+			rc = fdisk_reread_partition_table(cxt);
 		if (!rc)
 			rc = fdisk_deassign_device(cxt, 0);
 		/* fallthrough */
@@ -610,8 +666,13 @@ static int generic_menu_cb(struct fdisk_context **cxt0,
 		list_partition_types(cxt);
 		break;
 	case 'n':
-		rc = fdisk_add_partition(cxt, NULL, NULL);
+	{
+		size_t partno;
+		rc = fdisk_add_partition(cxt, NULL, &partno);
+		if (!rc)
+			rc = ask_for_wipe(cxt, partno);
 		break;
+	}
 	case 't':
 		change_partition_type(cxt);
 		break;
@@ -644,7 +705,7 @@ static int generic_menu_cb(struct fdisk_context **cxt0,
 
 /*
  * This is fdisk frontend for GPT specific libfdisk functions that
- * are not expported by generic libfdisk API.
+ * are not exported by generic libfdisk API.
  */
 static int gpt_menu_cb(struct fdisk_context **cxt0,
 		       const struct menu *menu __attribute__((__unused__)),
@@ -655,6 +716,7 @@ static int gpt_menu_cb(struct fdisk_context **cxt0,
 	struct fdisk_partition *pa = NULL;
 	size_t n;
 	int rc = 0;
+	uintmax_t length = 0;
 
 	assert(cxt);
 	assert(ent);
@@ -666,6 +728,12 @@ static int gpt_menu_cb(struct fdisk_context **cxt0,
 		switch (ent->key) {
 		case 'i':
 			return fdisk_set_disklabel_id(cxt);
+		case 'l':
+	                rc =  fdisk_ask_number(cxt, 1, fdisk_get_npartitions(cxt),
+	                                ~(uint32_t)0, _("New maximum entries"), &length);
+			if (rc)
+				return rc;
+			return fdisk_gpt_set_npartitions(cxt, (uint32_t) length);
 		case 'M':
 			mbr = fdisk_new_nested_context(cxt, "dos");
 			if (!mbr)
@@ -733,7 +801,7 @@ static int gpt_menu_cb(struct fdisk_context **cxt0,
 
 /*
  * This is fdisk frontend for MBR specific libfdisk functions that
- * are not expported by generic libfdisk API.
+ * are not exported by generic libfdisk API.
  */
 static int dos_menu_cb(struct fdisk_context **cxt0,
 		       const struct menu *menu __attribute__((__unused__)),
@@ -898,7 +966,7 @@ static int sgi_menu_cb(struct fdisk_context **cxt0,
 
 /*
  * This is fdisk frontend for BSD specific libfdisk functions that
- * are not expported by generic libfdisk API.
+ * are not exported by generic libfdisk API.
  */
 static int bsd_menu_cb(struct fdisk_context **cxt0,
 		       const struct menu *menu __attribute__((__unused__)),
@@ -988,22 +1056,25 @@ static int createlabel_menu_cb(struct fdisk_context **cxt0,
 			rc = fdisk_create_disklabel(cxt, "sgi");
 			break;
 		}
-		return rc;
+	} else {
+		switch (ent->key) {
+			case 'g':
+				rc = fdisk_create_disklabel(cxt, "gpt");
+				break;
+			case 'G':
+				rc = fdisk_create_disklabel(cxt, "sgi");
+				break;
+			case 'o':
+				rc = fdisk_create_disklabel(cxt, "dos");
+				break;
+			case 's':
+				rc = fdisk_create_disklabel(cxt, "sun");
+				break;
+		}
 	}
 
-	switch (ent->key) {
-		case 'g':
-			fdisk_create_disklabel(cxt, "gpt");
-			break;
-		case 'G':
-			fdisk_create_disklabel(cxt, "sgi");
-			break;
-		case 'o':
-			fdisk_create_disklabel(cxt, "dos");
-			break;
-		case 's':
-			fdisk_create_disklabel(cxt, "sun");
-			break;
-	}
+	if (rc == 0 && fdisk_get_collision(cxt))
+		follow_wipe_mode(cxt);
+
 	return rc;
 }
