@@ -16,6 +16,7 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <uuid.h>
 
 #include "c.h"
 #include "libfdisk.h"
@@ -378,6 +379,7 @@ struct fdisk_context {
 		     display_details : 1,	/* expert display mode */
 		     protect_bootbits : 1,	/* don't zeroize first sector */
 		     pt_collision : 1,		/* another PT detected by libblkid */
+		     no_disalogs : 1,		/* disable dialog-driven partititoning */
 		     listonly : 1;		/* list partition, nothing else */
 
 	char *collision;			/* name of already existing FS/PT */
@@ -398,6 +400,7 @@ struct fdisk_context {
 	struct fdisk_geometry user_geom;
 	unsigned long user_pyh_sector;
 	unsigned long user_log_sector;
+	unsigned long user_grain;
 
 	struct fdisk_label *label;	/* current label, pointer to labels[] */
 
@@ -412,6 +415,19 @@ struct fdisk_context {
 	struct fdisk_script	*script;	/* what we want to follow */
 };
 
+/* table */
+enum {
+	FDISK_DIFF_UNCHANGED = 0,
+	FDISK_DIFF_REMOVED,
+	FDISK_DIFF_ADDED,
+	FDISK_DIFF_MOVED,
+	FDISK_DIFF_RESIZED
+};
+extern int fdisk_diff_tables(struct fdisk_table *a, struct fdisk_table *b,
+				struct fdisk_iter *itr,
+				struct fdisk_partition **res, int *change);
+extern void fdisk_debug_print_table(struct fdisk_table *tb);
+
 
 /* context.c */
 extern int __fdisk_switch_label(struct fdisk_context *cxt,
