@@ -12,7 +12,6 @@
 #include <unistd.h>		/* write */
 #include <sys/ioctl.h>		/* ioctl */
 
-#include "nls.h"
 #include "blkdev.h"
 #include "bitops.h"
 
@@ -111,8 +110,6 @@ static int sun_probe_label(struct fdisk_context *cxt)
 {
 	struct fdisk_sun_label *sun;
 	struct sun_disklabel *sunlabel;
-	unsigned short *ush;
-	int csum;
 	int need_fixing = 0;
 
 	assert(cxt);
@@ -120,7 +117,7 @@ static int sun_probe_label(struct fdisk_context *cxt)
 	assert(fdisk_is_label(cxt, SUN));
 
 	/* map first sector to header */
-	sun = (struct fdisk_sun_label *) cxt->label;
+	sun = self_label(cxt);
 	sun->header = (struct sun_disklabel *) cxt->firstsector;
 	sunlabel = sun->header;
 
@@ -129,11 +126,7 @@ static int sun_probe_label(struct fdisk_context *cxt)
 		return 0;		/* failed */
 	}
 
-	ush = ((unsigned short *) (sunlabel + 1)) - 1;
-	for (csum = 0; ush >= (unsigned short *)sunlabel;)
-		csum ^= *ush--;
-
-	if (csum) {
+	if (sun_pt_checksum(sunlabel)) {
 		fdisk_warnx(cxt, _("Detected sun disklabel with wrong checksum. "
 			      "Probably you'll have to set all the values, "
 			      "e.g. heads, sectors, cylinders and partitions "
@@ -146,6 +139,10 @@ static int sun_probe_label(struct fdisk_context *cxt)
 	cxt->geom.cylinders = be16_to_cpu(sunlabel->ncyl);
 	cxt->geom.sectors = be16_to_cpu(sunlabel->nsect);
 
+	/* we have on label geom, but user has to win */
+	if (fdisk_has_user_device_geometry(cxt))
+		fdisk_apply_user_device_properties(cxt);
+
 	if (be32_to_cpu(sunlabel->vtoc.version) != SUN_VTOC_VERSION) {
 		fdisk_warnx(cxt, _("Detected sun disklabel with wrong version [%d]."),
 			be32_to_cpu(sunlabel->vtoc.version));
@@ -168,12 +165,8 @@ static int sun_probe_label(struct fdisk_context *cxt)
 		sunlabel->vtoc.version = cpu_to_be32(SUN_VTOC_VERSION);
 		sunlabel->vtoc.sanity = cpu_to_be32(SUN_VTOC_SANITY);
 		sunlabel->vtoc.nparts = cpu_to_be16(SUN_MAXPARTITIONS);
-
-		ush = (unsigned short *)sunlabel;
-		csum = 0;
-		while(ush < (unsigned short *)(&sunlabel->csum))
-			csum ^= *ush++;
-		sunlabel->csum = csum;
+		sunlabel->csum = 0;
+		sunlabel->csum = sun_pt_checksum(sunlabel);
 
 		fdisk_label_set_changed(cxt->label, 1);
 	}
@@ -213,7 +206,7 @@ static int sun_create_disklabel(struct fdisk_context *cxt)
 	if (rc)
 		return rc;
 
-	sun = (struct fdisk_sun_label *) cxt->label;
+	sun = self_label(cxt);
 	sun->header = (struct sun_disklabel *) cxt->firstsector;
 
 	sunlabel = sun->header;
@@ -283,13 +276,8 @@ static int sun_create_disklabel(struct fdisk_context *cxt)
 			  SUN_TAG_WHOLEDISK);
 	}
 
-	{
-		unsigned short *ush = (unsigned short *)sunlabel;
-		unsigned short csum = 0;
-		while(ush < (unsigned short *)(&sunlabel->csum))
-			csum ^= *ush++;
-		sunlabel->csum = csum;
-	}
+	sunlabel->csum = 0;
+	sunlabel->csum = sun_pt_checksum(sunlabel);
 
 	fdisk_label_set_changed(cxt->label, 1);
 	cxt->label->nparts_cur = count_used_partitions(cxt);
@@ -338,6 +326,7 @@ static void fetch_sun(struct fdisk_context *cxt,
 	struct sun_disklabel *sunlabel;
 	int continuous = 1;
 	size_t i;
+	int sectors_per_cylinder = cxt->geom.heads * cxt->geom.sectors;
 
 	assert(cxt);
 	assert(cxt);
@@ -347,7 +336,7 @@ static void fetch_sun(struct fdisk_context *cxt,
 	sunlabel = self_disklabel(cxt);
 
 	*start = 0;
-	*stop = cxt->geom.cylinders * cxt->geom.heads * cxt->geom.sectors;
+	*stop = cxt->geom.cylinders * sectors_per_cylinder;
 
 	for (i = 0; i < cxt->label->nparts_max; i++) {
 		struct sun_partition *part = &sunlabel->partitions[i];
@@ -357,12 +346,16 @@ static void fetch_sun(struct fdisk_context *cxt,
 		    be16_to_cpu(info->id) != SUN_TAG_UNASSIGNED &&
 		    be16_to_cpu(info->id) != SUN_TAG_WHOLEDISK) {
 			starts[i] = be32_to_cpu(part->start_cylinder) *
-				     cxt->geom.heads * cxt->geom.sectors;
+				     sectors_per_cylinder;
 			lens[i] = be32_to_cpu(part->num_sectors);
 			if (continuous) {
-				if (starts[i] == *start)
+				if (starts[i] == *start) {
 					*start += lens[i];
-				else if (starts[i] + lens[i] >= *stop)
+					int remained_sectors = *start % sectors_per_cylinder;
+					if (remained_sectors) {
+						*start += sectors_per_cylinder - remained_sectors;
+					}
+				} else if (starts[i] + lens[i] >= *stop)
 					*stop = starts[i];
 				else
 					continuous = 0;
@@ -541,6 +534,11 @@ static int sun_add_partition(
 	} else {
 		struct fdisk_ask *ask;
 
+		if (n == 2)
+			fdisk_info(cxt, _("It is highly recommended that the "
+					   "third partition covers the whole disk "
+					   "and is of type `Whole disk'"));
+
 		snprintf(mesg, sizeof(mesg), _("First %s"),
 				fdisk_get_unit(cxt, FDISK_SINGULAR));
 		for (;;) {
@@ -555,6 +553,10 @@ static int sun_add_partition(
 				fdisk_ask_number_set_low(ask,     0);	/* minimal */
 				fdisk_ask_number_set_default(ask, 0);	/* default */
 				fdisk_ask_number_set_high(ask,    0);	/* maximal */
+			} else if (n == 2) {
+				fdisk_ask_number_set_low(ask,     0);				/* minimal */
+				fdisk_ask_number_set_default(ask, 0);                           /* default */
+				fdisk_ask_number_set_high(ask,    fdisk_scround(cxt, stop));    /* maximal */
 			} else {
 				fdisk_ask_number_set_low(ask,     fdisk_scround(cxt, start));	/* minimal */
 				fdisk_ask_number_set_default(ask, fdisk_scround(cxt, start));	/* default */
@@ -569,6 +571,19 @@ static int sun_add_partition(
 			if (fdisk_use_cylinders(cxt))
 				first *= fdisk_get_units_per_sector(cxt);
 
+			if (!fdisk_use_cylinders(cxt)) {
+				/* Starting sector has to be properly aligned */
+				int cs = cxt->geom.heads * cxt->geom.sectors;
+				int x = first % cs;
+
+				if (x) {
+					fdisk_info(cxt, _("Aligning the first sector from %u to %u "
+							  "to be on cylinder boundary."),
+							first, first + cs - x);
+					first += cs - x;
+				}
+			}
+
 			/* ewt asks to add: "don't start a partition at cyl 0"
 			   However, edmundo@rano.demon.co.uk writes:
 			   "In addition to having a Sun partition table, to be able to
@@ -596,24 +611,6 @@ static int sun_add_partition(
 		}
 	}
 
-	if (n == 2 && first != 0)
-		fdisk_warnx(cxt, _("It is highly recommended that the "
-				   "third partition covers the whole disk "
-				   "and is of type `Whole disk'"));
-
-	if (!fdisk_use_cylinders(cxt)) {
-		/* Starting sector has to be properly aligned */
-		int cs = cxt->geom.heads * cxt->geom.sectors;
-		int x = first % cs;
-
-		if (x) {
-			fdisk_info(cxt, _("Aligning the first sector from %u to %u "
-					  "to be on cylinder boundary."),
-					first, first + cs - x);
-			first += cs - x;
-		}
-	}
-
 	stop = cxt->geom.cylinders * cxt->geom.heads * cxt->geom.sectors;	/* ancient */
 	stop2 = stop;
 	for (i = 0; i < cxt->label->nparts_max; i++) {
@@ -976,8 +973,6 @@ int fdisk_sun_set_pcylcount(struct fdisk_context *cxt)
 static int sun_write_disklabel(struct fdisk_context *cxt)
 {
 	struct sun_disklabel *sunlabel;
-	unsigned short *ush;
-	unsigned short csum = 0;
 	const size_t sz = sizeof(struct sun_disklabel);
 
 	assert(cxt);
@@ -996,11 +991,9 @@ static int sun_write_disklabel(struct fdisk_context *cxt)
 		sunlabel->ncyl = a - b;
 	}
 
-	ush = (unsigned short *) sunlabel;
+	sunlabel->csum = 0;
+	sunlabel->csum = sun_pt_checksum(sunlabel);
 
-	while(ush < (unsigned short *)(&sunlabel->csum))
-		csum ^= *ush++;
-	sunlabel->csum = csum;
 	if (lseek(cxt->dev_fd, 0, SEEK_SET) < 0)
 		return -errno;
 	if (write_all(cxt->dev_fd, sunlabel, sz) != 0)
@@ -1117,7 +1110,7 @@ static const struct fdisk_field sun_fields[] =
 	{ FDISK_FIELD_ATTR,	N_("Flags"),	  0,	FDISK_FIELDFL_NUMBER }
 };
 
-const struct fdisk_label_operations sun_operations =
+static const struct fdisk_label_operations sun_operations =
 {
 	.probe		= sun_probe_label,
 	.write		= sun_write_disklabel,
