@@ -105,7 +105,9 @@ int mnt_update_set_filename(struct libmnt_update *upd, const char *filename,
 
 	/* detect tab filename -- /etc/mtab or /run/mount/utab
 	 */
+#ifdef USE_LIBMOUNT_SUPPORT_MTAB
 	mnt_has_regular_mtab(&path, &rw);
+#endif
 	if (!rw) {
 		path = NULL;
 		mnt_has_regular_utab(&path, &rw);
@@ -289,7 +291,7 @@ static int utab_new_entry(struct libmnt_update *upd, struct libmnt_fs *fs,
 			  unsigned long mountflags)
 {
 	int rc = 0;
-	const char *o = NULL, *a = NULL;
+	const char *o, *a;
 	char *u = NULL;
 
 	assert(fs);
@@ -694,7 +696,7 @@ static int update_add_entry(struct libmnt_update *upd, struct libmnt_lock *lc)
 	if (lc)
 		rc = mnt_lock_file(lc);
 	if (rc)
-		return rc;
+		return -MNT_ERR_LOCK;
 
 	tb = __mnt_new_table_from_file(upd->filename,
 			upd->userspace_only ? MNT_FMT_UTAB : MNT_FMT_MTAB);
@@ -720,7 +722,7 @@ static int update_remove_entry(struct libmnt_update *upd, struct libmnt_lock *lc
 	if (lc)
 		rc = mnt_lock_file(lc);
 	if (rc)
-		return rc;
+		return -MNT_ERR_LOCK;
 
 	tb = __mnt_new_table_from_file(upd->filename,
 			upd->userspace_only ? MNT_FMT_UTAB : MNT_FMT_MTAB);
@@ -749,7 +751,7 @@ static int update_modify_target(struct libmnt_update *upd, struct libmnt_lock *l
 	if (lc)
 		rc = mnt_lock_file(lc);
 	if (rc)
-		return rc;
+		return -MNT_ERR_LOCK;
 
 	tb = __mnt_new_table_from_file(upd->filename,
 			upd->userspace_only ? MNT_FMT_UTAB : MNT_FMT_MTAB);
@@ -786,7 +788,7 @@ static int update_modify_options(struct libmnt_update *upd, struct libmnt_lock *
 	if (lc)
 		rc = mnt_lock_file(lc);
 	if (rc)
-		return rc;
+		return -MNT_ERR_LOCK;
 
 	tb = __mnt_new_table_from_file(upd->filename,
 			upd->userspace_only ? MNT_FMT_UTAB : MNT_FMT_MTAB);
@@ -883,10 +885,13 @@ int mnt_update_already_done(struct libmnt_update *upd, struct libmnt_lock *lc)
 	}
 	if (lc && upd->userspace_only)
 		mnt_lock_use_simplelock(lc, TRUE);	/* use flock */
-	if (lc)
+	if (lc) {
 		rc = mnt_lock_file(lc);
-	if (rc)
-		goto done;
+		if (rc) {
+			rc = -MNT_ERR_LOCK;
+			goto done;
+		}
+	}
 
 	tb = __mnt_new_table_from_file(upd->filename,
 			upd->userspace_only ? MNT_FMT_UTAB : MNT_FMT_MTAB);
