@@ -53,7 +53,7 @@ static int string_replace(char *from, char *to, char *s, char *orig, char **newn
 	return 0;
 }
 
-static int do_symlink(char *from, char *to, char *s, int verbose, int noact)
+static int do_symlink(char *from, char *to, char *s, int verbose, int noact, int nooverwrite)
 {
 	char *newname = NULL, *target = NULL;
 	int ret = 1;
@@ -76,24 +76,36 @@ static int do_symlink(char *from, char *to, char *s, int verbose, int noact)
 	target[sb.st_size] = '\0';
 	if (string_replace(from, to, target, target, &newname))
 		ret = 0;
-	else if (!noact && 0 > unlink(s)) {
-		warn(_("%s: unlink failed"), s);
-		ret = 2;
-	} else if (!noact && symlink(newname, s) != 0) {
-		warn(_("%s: symlinking to %s failed"), s, newname);
-		ret = 2;
+
+	if (ret == 1 && nooverwrite && lstat(newname, &sb) == 0) {
+		if (verbose)
+			printf(_("Skipping existing link: `%s'\n"), newname);
+
+		ret = 0;
+	}
+
+	if (ret == 1) {
+		if (!noact && 0 > unlink(s)) {
+			warn(_("%s: unlink failed"), s);
+			ret = 2;
+		} else if (!noact && symlink(newname, s) != 0) {
+			warn(_("%s: symlinking to %s failed"), s, newname);
+			ret = 2;
+		}
 	}
 	if (verbose && (noact || ret == 1))
-		printf("%s: `%s' -> `%s'\n", s, target, newname);
+		if (verbose)
+			printf("%s: `%s' -> `%s'\n", s, target, newname);
 	free(newname);
 	free(target);
 	return ret;
 }
 
-static int do_file(char *from, char *to, char *s, int verbose, int noact)
+static int do_file(char *from, char *to, char *s, int verbose, int noact, int nooverwrite)
 {
 	char *newname = NULL, *file=NULL;
 	int ret = 1;
+	struct stat sb;
 
 	if (strchr(from, '/') == NULL && strchr(to, '/') == NULL)
 		file = strrchr(s, '/');
@@ -101,6 +113,10 @@ static int do_file(char *from, char *to, char *s, int verbose, int noact)
 		file = s;
 	if (string_replace(from, to, file, s, &newname))
 		return 0;
+	if (nooverwrite && stat(newname, &sb) == 0) {
+		printf(_("Skipping existing file: `%s'\n"), newname);
+		ret = 0;
+	}
 	else if (!noact && rename(s, newname) != 0) {
 		warn(_("%s: rename to %s failed"), s, newname);
 		ret = 2;
@@ -111,8 +127,9 @@ static int do_file(char *from, char *to, char *s, int verbose, int noact)
 	return ret;
 }
 
-static void __attribute__ ((__noreturn__)) usage(FILE * out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fputs(USAGE_HEADER, out);
 	fprintf(out,
 	      _(" %s [options] <expression> <replacement> <file>...\n"),
@@ -122,27 +139,28 @@ static void __attribute__ ((__noreturn__)) usage(FILE * out)
 	fputs(_("Rename files.\n"), out);
 
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" -v, --verbose    explain what is being done\n"), out);
-	fputs(_(" -s, --symlink    act on the target of symlinks\n"), out);
-	fputs(_(" -n, --no-act     do not make any changes\n"), out);
+	fputs(_(" -v, --verbose       explain what is being done\n"), out);
+	fputs(_(" -s, --symlink       act on the target of symlinks\n"), out);
+	fputs(_(" -n, --no-act        do not make any changes\n"), out);
+	fputs(_(" -o, --no-overwrite  don't overwrite existing files\n"), out);
 	fputs(USAGE_SEPARATOR, out);
-	fputs(USAGE_HELP, out);
-	fputs(USAGE_VERSION, out);
-	fprintf(out, USAGE_MAN_TAIL("rename(1)"));
-	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	printf(USAGE_HELP_OPTIONS(21));
+	printf(USAGE_MAN_TAIL("rename(1)"));
+	exit(EXIT_SUCCESS);
 }
 
 int main(int argc, char **argv)
 {
 	char *from, *to;
-	int i, c, ret = 0, verbose = 0, noact = 0;
-	int (*do_rename)(char *from, char *to, char *s, int verbose, int noact) = do_file;
+	int i, c, ret = 0, verbose = 0, noact = 0, nooverwrite = 0;
+	int (*do_rename)(char *from, char *to, char *s, int verbose, int noact, int nooverwrite) = do_file;
 
 	static const struct option longopts[] = {
 		{"verbose", no_argument, NULL, 'v'},
 		{"version", no_argument, NULL, 'V'},
 		{"help", no_argument, NULL, 'h'},
 		{"no-act", no_argument, NULL, 'n'},
+		{"no-overwrite", no_argument, NULL, 'o'},
 		{"symlink", no_argument, NULL, 's'},
 		{NULL, 0, NULL, 0}
 	};
@@ -152,11 +170,14 @@ int main(int argc, char **argv)
 	textdomain(PACKAGE);
 	atexit(close_stdout);
 
-	while ((c = getopt_long(argc, argv, "vsVhn", longopts, NULL)) != -1)
+	while ((c = getopt_long(argc, argv, "vsVhno", longopts, NULL)) != -1)
 		switch (c) {
 		case 'n':
 			noact = 1;
 			/* fallthrough */
+		case 'o':
+			nooverwrite = 1;
+                        break;
 		case 'v':
 			verbose = 1;
 			break;
@@ -167,8 +188,7 @@ int main(int argc, char **argv)
 			printf(UTIL_LINUX_VERSION);
 			return EXIT_SUCCESS;
 		case 'h':
-			usage(stdout);
-			/* fallthrough */
+			usage();
 		default:
 			errtryhelp(EXIT_FAILURE);
 		}
@@ -178,14 +198,17 @@ int main(int argc, char **argv)
 
 	if (argc < 3) {
 		warnx(_("not enough arguments"));
-		usage(stderr);
+		errtryhelp(EXIT_FAILURE);
 	}
 
 	from = argv[0];
 	to = argv[1];
 
+	if (!strcmp(from, to))
+		return RENAME_EXIT_NOTHING;
+
 	for (i = 2; i < argc; i++)
-		ret |= do_rename(from, to, argv[i], verbose, noact);
+		ret |= do_rename(from, to, argv[i], verbose, noact, nooverwrite);
 
 	switch (ret) {
 	case 0:
