@@ -184,17 +184,17 @@ static int fstrim_all(struct fstrim_range *rangetpl, int verbose)
 
 	itr = mnt_new_iter(MNT_ITER_BACKWARD);
 	if (!itr)
-		err(MOUNT_EX_FAIL, _("failed to initialize libmount iterator"));
+		err(MNT_EX_FAIL, _("failed to initialize libmount iterator"));
 
 	tab = mnt_new_table_from_file(_PATH_PROC_MOUNTINFO);
 	if (!tab)
-		err(MOUNT_EX_FAIL, _("failed to parse %s"), _PATH_PROC_MOUNTINFO);
+		err(MNT_EX_FAIL, _("failed to parse %s"), _PATH_PROC_MOUNTINFO);
 
 	/* de-duplicate by mountpoints */
 	mnt_table_uniq_fs(tab, 0, uniq_fs_target_cmp);
 
-	/* de-duplicate by source and root */
-	mnt_table_uniq_fs(tab, 0, uniq_fs_source_cmp);
+	/* de-duplicate by source */
+	mnt_table_uniq_fs(tab, MNT_UNIQ_FORWARD, uniq_fs_source_cmp);
 
 	while (mnt_table_next_fs(tab, itr, &fs) == 0) {
 		const char *src = mnt_fs_get_srcpath(fs),
@@ -237,15 +237,16 @@ static int fstrim_all(struct fstrim_range *rangetpl, int verbose)
 	mnt_free_iter(itr);
 
 	if (cnt && cnt == cnt_err)
-		return MOUNT_EX_FAIL;		/* all failed */
+		return MNT_EX_FAIL;		/* all failed */
 	if (cnt && cnt_err)
-		return MOUNT_EX_SOMEOK;		/* some ok */
+		return MNT_EX_SOMEOK;		/* some ok */
 
 	return EXIT_SUCCESS;
 }
 
-static void __attribute__((__noreturn__)) usage(FILE *out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fputs(USAGE_HEADER, out);
 	fprintf(out,
 	      _(" %s [options] <mount point>\n"), program_invocation_short_name);
@@ -261,10 +262,9 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(_(" -v, --verbose       print number of discarded bytes\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
-	fputs(USAGE_HELP, out);
-	fputs(USAGE_VERSION, out);
-	fprintf(out, USAGE_MAN_TAIL("fstrim(8)"));
-	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	printf(USAGE_HELP_OPTIONS(21));
+	printf(USAGE_MAN_TAIL("fstrim(8)"));
+	exit(EXIT_SUCCESS);
 }
 
 int main(int argc, char **argv)
@@ -274,14 +274,14 @@ int main(int argc, char **argv)
 	struct fstrim_range range;
 
 	static const struct option longopts[] = {
-	    { "all",       0, 0, 'a' },
-	    { "help",      0, 0, 'h' },
-	    { "version",   0, 0, 'V' },
-	    { "offset",    1, 0, 'o' },
-	    { "length",    1, 0, 'l' },
-	    { "minimum",   1, 0, 'm' },
-	    { "verbose",   0, 0, 'v' },
-	    { NULL,        0, 0, 0 }
+	    { "all",       no_argument,       NULL, 'a' },
+	    { "help",      no_argument,       NULL, 'h' },
+	    { "version",   no_argument,       NULL, 'V' },
+	    { "offset",    required_argument, NULL, 'o' },
+	    { "length",    required_argument, NULL, 'l' },
+	    { "minimum",   required_argument, NULL, 'm' },
+	    { "verbose",   no_argument,       NULL, 'v' },
+	    { NULL, 0, NULL, 0 }
 	};
 
 	setlocale(LC_ALL, "");
@@ -298,7 +298,7 @@ int main(int argc, char **argv)
 			all = 1;
 			break;
 		case 'h':
-			usage(stdout);
+			usage();
 			break;
 		case 'V':
 			printf(UTIL_LINUX_VERSION);
@@ -319,7 +319,7 @@ int main(int argc, char **argv)
 			verbose = 1;
 			break;
 		default:
-			usage(stderr);
+			errtryhelp(EXIT_FAILURE);
 			break;
 		}
 	}
@@ -332,7 +332,7 @@ int main(int argc, char **argv)
 
 	if (optind != argc) {
 		warnx(_("unexpected number of arguments"));
-		usage(stderr);
+		errtryhelp(EXIT_FAILURE);
 	}
 
 	if (all)
