@@ -1,7 +1,6 @@
 #ifndef HWCLOCK_CLOCK_H
 #define HWCLOCK_CLOCK_H
 
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -10,17 +9,11 @@
 
 #include "c.h"
 
-enum {
-	RTC_BUSYWAIT_OK = 0,
-	RTC_BUSYWAIT_FAILED,
-	RTC_BUSYWAIT_TIMEOUT
-};
-
 struct hwclock_control {
 	char *date_opt;
 	char *adj_file_name;
 #if defined(__linux__) && defined(__alpha__)
-	unsigned long epoch_option;
+	char *epoch_option;
 #endif
 #ifdef __linux__
 	char *rtc_dev_name;
@@ -60,11 +53,8 @@ struct clock_ops {
 extern struct clock_ops *probe_for_cmos_clock(void);
 extern struct clock_ops *probe_for_rtc_clock(const struct hwclock_control *ctl);
 
-typedef int bool;
-
 /* hwclock.c */
 extern int debug;
-extern unsigned long epoch_option;
 extern double time_diff(struct timeval subtrahend, struct timeval subtractor);
 
 /* rtc.c */
@@ -73,6 +63,7 @@ extern int get_epoch_rtc(const struct hwclock_control *ctl, unsigned long *epoch
 extern int set_epoch_rtc(const struct hwclock_control *ctl);
 #endif
 
-extern void hwclock_exit(const struct hwclock_control *ctl, int status);
+extern void __attribute__((__noreturn__))
+hwclock_exit(const struct hwclock_control *ctl, int status);
 
 #endif				/* HWCLOCK_CLOCK_H */
