@@ -60,21 +60,19 @@
 #include <termios.h>
 #include <unistd.h>
 
-#ifndef NCURSES_CONST
-# define NCURSES_CONST const	/* define before including term.h */
+#if defined(HAVE_NCURSESW_TERM_H)
+# include <ncursesw/term.h>
+#elif defined(HAVE_NCURSES_TERM_H)
+# include <ncurses/term.h>
+#elif defined(HAVE_TERM_H)
+# include <term.h>
 #endif
-#ifdef HAVE_NCURSES_H
-# include <ncurses.h>
-#elif defined(HAVE_NCURSES_NCURSES_H)
-# include <ncurses/ncurses.h>
-#endif
-/* must include after ncurses.h */
-#include <term.h>
 
 #ifdef HAVE_LINUX_TIOCL_H
 # include <linux/tiocl.h>
 #endif
 
+#include "all-io.h"
 #include "c.h"
 #include "closestream.h"
 #include "nls.h"
@@ -175,14 +173,14 @@ struct setterm_control {
 	int opt_rt_len;		/* regular tab length */
 	int opt_tb_array[TABS_MAX + 1];	/* array for tab list */
 	/* colors */
-	int opt_fo_color:4, opt_ba_color:4, opt_ul_color:4, opt_hb_color:4;
+	unsigned int opt_fo_color:4, opt_ba_color:4, opt_ul_color:4, opt_hb_color:4;
 	/* boolean options */
 	unsigned int opt_cu_on:1, opt_li_on:1, opt_bo_on:1, opt_hb_on:1,
 	    opt_bl_on:1, opt_re_on:1, opt_un_on:1, opt_rep_on:1,
 	    opt_appck_on:1, opt_invsc_on:1, opt_msg_on:1, opt_cl_all:1,
 	    vcterm:1;
 	/* Option flags.  Set when an option is invoked. */
-	uint64_t opt_term:1, opt_reset:1, opt_initialize:1, opt_cursor:1,
+	uint64_t opt_term:1, opt_reset:1, opt_resize:1, opt_initialize:1, opt_cursor:1,
 	    opt_linewrap:1, opt_default:1, opt_foreground:1,
 	    opt_background:1, opt_bold:1, opt_blink:1, opt_reverse:1,
 	    opt_underline:1, opt_store:1, opt_clear:1, opt_blank:1,
@@ -373,8 +371,9 @@ static int parse_bfreq(char **av, char *oa, int *oi)
 	return strtos32_or_err(arg, _("argument error"));
 }
 
-static void __attribute__((__noreturn__)) usage(FILE *out)
+static void __attribute__((__noreturn__)) usage(void)
 {
+	FILE *out = stdout;
 	fputs(USAGE_HEADER, out);
 	fprintf(out,
 	      _(" %s [options]\n"), program_invocation_short_name);
@@ -385,6 +384,7 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" --term          <terminal_name>   override TERM environment variable\n"), out);
 	fputs(_(" --reset                           reset terminal to power-on state\n"), out);
+	fputs(_(" --resize                          reset terminal rows and columns\n"), out);
 	fputs(_(" --initialize                      display init string, and use default settings\n"), out);
 	fputs(_(" --default                         use default terminal settings\n"), out);
 	fputs(_(" --store                           save current terminal settings as default\n"), out);
@@ -418,10 +418,11 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(_(" --powerdown     [0-60]            set vesa powerdown interval in minutes\n"), out);
 	fputs(_(" --blength       [0-2000]          duration of the bell in milliseconds\n"), out);
 	fputs(_(" --bfreq         <number>          bell frequency in Hertz\n"), out);
-	fputs(_(" --version                         show version information and exit\n"), out);
-	fputs(_(" --help                            display this help and exit\n"), out);
-	fprintf(out, USAGE_MAN_TAIL("setterm(1)"));
-	exit(out == stderr ? EXIT_FAILURE : EXIT_SUCCESS);
+	printf( " --help                            %s\n", USAGE_OPTSTR_HELP);
+	printf( " --version                         %s\n", USAGE_OPTSTR_VERSION);
+
+	printf(USAGE_MAN_TAIL("setterm(1)"));
+	exit(EXIT_SUCCESS);
 }
 
 static int __attribute__((__pure__)) set_opt_flag(int opt)
@@ -437,6 +438,7 @@ static void parse_option(struct setterm_control *ctl, int ac, char **av)
 	enum {
 		OPT_TERM = CHAR_MAX + 1,
 		OPT_RESET,
+		OPT_RESIZE,
 		OPT_INITIALIZE,
 		OPT_CURSOR,
 		OPT_REPEAT,
@@ -474,6 +476,7 @@ static void parse_option(struct setterm_control *ctl, int ac, char **av)
 	static const struct option longopts[] = {
 		{"term", required_argument, NULL, OPT_TERM},
 		{"reset", no_argument, NULL, OPT_RESET},
+		{"resize", no_argument, NULL, OPT_RESIZE},
 		{"initialize", no_argument, NULL, OPT_INITIALIZE},
 		{"cursor", required_argument, NULL, OPT_CURSOR},
 		{"repeat", required_argument, NULL, OPT_REPEAT},
@@ -527,6 +530,9 @@ static void parse_option(struct setterm_control *ctl, int ac, char **av)
 		case OPT_RESET:
 			ctl->opt_reset = set_opt_flag(ctl->opt_reset);
 			break;
+		case OPT_RESIZE:
+			ctl->opt_resize = set_opt_flag(ctl->opt_resize);
+			break;
 		case OPT_INITIALIZE:
 			ctl->opt_initialize = set_opt_flag(ctl->opt_initialize);
 			break;
@@ -668,9 +674,9 @@ static void parse_option(struct setterm_control *ctl, int ac, char **av)
 			printf(UTIL_LINUX_VERSION);
 			exit(EXIT_SUCCESS);
 		case OPT_HELP:
-			usage(stdout);
+			usage();
 		default:
-			usage(stderr);
+			errtryhelp(EXIT_FAILURE);
 		}
 	}
 }
@@ -768,7 +774,7 @@ static void screendump(struct setterm_control *ctl)
 		ctl->opt_sn_name = "screen.dump";
 	out = fopen(ctl->opt_sn_name, ctl->opt_snap ? "w" : "a");
 	if (!out)
-		err(EXIT_DUMPFILE, _("can not open dump file %s for output"), ctl->opt_sn_name);
+		err(EXIT_DUMPFILE, _("cannot open dump file %s for output"), ctl->opt_sn_name);
 	/* determine snapshot size */
 	if (read(fd, header, 4) != 4)
 		err(EXIT_DUMPFILE, _("cannot read %s"), ctl->in_device);
@@ -815,6 +821,104 @@ static int vc_only(struct setterm_control *ctl, const char *err)
 	return ctl->vcterm;
 }
 
+static void tty_raw(struct termios *saved_attributes, int *saved_fl)
+{
+	struct termios tattr;
+
+	fcntl(STDIN_FILENO, F_GETFL, saved_fl);
+	tcgetattr(STDIN_FILENO, saved_attributes);
+	fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
+	memcpy(&tattr, saved_attributes, sizeof(struct termios));
+	tattr.c_lflag &= ~(ICANON | ECHO);
+	tattr.c_cc[VMIN] = 1;
+	tattr.c_cc[VTIME] = 0;
+	tcsetattr(STDIN_FILENO, TCSAFLUSH, &tattr);
+}
+
+static void tty_restore(struct termios *saved_attributes, int *saved_fl)
+{
+	fcntl(STDIN_FILENO, F_SETFL, *saved_fl);
+	tcsetattr(STDIN_FILENO, TCSANOW, saved_attributes);
+}
+
+static int select_wait(void)
+{
+	struct timeval tv;
+	fd_set set;
+	int ret;
+
+	FD_ZERO(&set);
+	FD_SET(STDIN_FILENO, &set);
+	tv.tv_sec = 10;
+	tv.tv_usec = 0;
+	while ((ret = select(1, &set, NULL, NULL, &tv)) < 0) {
+		if (errno == EINTR)
+			continue;
+		err(EXIT_FAILURE, _("select failed"));
+	}
+	return ret;
+}
+
+static int resizetty(void)
+{
+	/* 
+	 * \e7        Save current state (cursor coordinates, attributes,
+	 *                character sets pointed at by G0, G1).
+	 * \e[r       Set scrolling region; parameters are top and bottom row.
+	 * \e[32766E  Move cursor down 32766 (INT16_MAX - 1) rows.
+	 * \e[32766C  Move cursor right 32766 columns.
+	 * \e[6n      Report cursor position.
+	 * \e8        Restore state most recently saved by \e7.
+	 */
+	static const char *getpos = "\e7\e[r\e[32766E\e[32766C\e[6n\e8";
+	char retstr[32];
+	int row, col;
+	size_t pos;
+	ssize_t rc;
+	struct winsize ws;
+	struct termios saved_attributes;
+	int saved_fl;
+
+	if (!isatty(STDIN_FILENO))
+		errx(EXIT_FAILURE, _("stdin does not refer to a terminal"));
+
+	tty_raw(&saved_attributes, &saved_fl);
+	if (write_all(STDIN_FILENO, getpos, strlen(getpos)) < 0) {
+		warn(_("write failed"));
+		tty_restore(&saved_attributes, &saved_fl);
+		return 1;
+	}
+	for (pos = 0; pos < sizeof(retstr) - 1;) {
+		if (0 == select_wait())
+			break;
+		if ((rc =
+		     read(STDIN_FILENO, retstr + pos,
+			  sizeof(retstr) - 1 - pos)) < 0) {
+			if (errno == EINTR)
+				continue;
+			warn(_("read failed"));
+			tty_restore(&saved_attributes, &saved_fl);
+			return 1;
+		}
+		pos += rc;
+		if (retstr[pos - 1] == 'R')
+			break;
+	}
+	retstr[pos] = 0;
+	tty_restore(&saved_attributes, &saved_fl);
+	rc = sscanf(retstr, "\033[%d;%dR", &row, &col);
+	if (rc != 2) {
+		warnx(_("invalid cursor position: %s"), retstr);
+		return 1;
+	}
+	memset(&ws, 0, sizeof(struct winsize));
+	ioctl(STDIN_FILENO, TIOCGWINSZ, &ws);
+	ws.ws_row = row;
+	ws.ws_col = col;
+	ioctl(STDIN_FILENO, TIOCSWINSZ, &ws);
+	return 0;
+}
+
 static void perform_sequence(struct setterm_control *ctl)
 {
 	int result;
@@ -823,6 +927,11 @@ static void perform_sequence(struct setterm_control *ctl)
 	if (ctl->opt_reset)
 		putp(ti_entry("rs1"));
 
+	/* -resize. */
+	if (ctl->opt_resize)
+		if (resizetty())
+			warnx(_("reset failed"));
+
 	/* -initialize. */
 	if (ctl->opt_initialize)
 		putp(ti_entry("is2"));
@@ -1056,16 +1165,17 @@ static void init_terminal(struct setterm_control *ctl)
 
 int main(int argc, char **argv)
 {
-	struct setterm_control ctl = { 0 };
+	struct setterm_control ctl = { NULL };
 
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 	atexit(close_stdout);
 
-	if (argc < 2)
-		usage(stderr);
-
+	if (argc < 2) {
+		warnx(_("bad usage"));
+		errtryhelp(EXIT_FAILURE);
+	}
 	parse_option(&ctl, argc, argv);
 	init_terminal(&ctl);
 	perform_sequence(&ctl);
