@@ -52,6 +52,7 @@
 #include "ttyutils.h"
 #include "strv.h"
 #include "optutils.h"
+#include "mbsalign.h"
 
 #include "libsmartcols.h"
 
@@ -458,8 +459,18 @@ static int read_input(struct column_control *ctl, FILE *fp)
 			continue;
 
 		wcs = mbs_to_wcs(str);
-		if (!wcs)
-			err(EXIT_FAILURE, _("read failed"));
+		if (!wcs) {
+			/*
+			 * Convert broken sequences to \x<hex> and continue.
+			 */
+			size_t tmpsz = 0;
+			char *tmp = mbs_invalid_encode(str, &tmpsz);
+
+			if (!tmp)
+				err(EXIT_FAILURE, _("read failed"));
+			wcs = mbs_to_wcs(tmp);
+			free(tmp);
+		}
 
 		switch (ctl->mode) {
 		case COLUMN_MODE_TABLE:
@@ -590,6 +601,7 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -s, --separator <string>         possible table delimiters\n"), out);
 	fputs(_(" -x, --fillrows                   fill rows before columns\n"), out);
 
+
 	fputs(USAGE_SEPARATOR, out);
 	printf(USAGE_HELP_OPTIONS(34));
 	printf(USAGE_MAN_TAIL("column(1)"));
